<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	QuickPay
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppQuickpay extends PayplansAppPayment
{
	
	//  quickpay status-codes
	const QUICKPAY_STATUS_20000  =  20000; // Approved
	const QUICKPAY_STATUS_40000  =  40000; // Rejected by acquirer
	const QUICKPAY_STATUS_40001  =  40001; //Request Data Error
	const QUICKPAY_STATUS_50000  =  50000; // Gateway Error
	const QUICKPAY_STATUS_50300  =  50300; //Communications Error (with Acquirer)

	
	//quickpay transaction state
	const QUICKPAY_TRANSACTION_STATE_INITIAL	 = 'initial';
	const QUICKPAY_TRANSACTION_STATE_PENDING	 = 'pending';
	const QUICKPAY_TRANSACTION_STATE_NEW	 = 'new';
	const QUICKPAY_TRANSACTION_STATE_REJECTED	 = 'rejected';
	const QUICKPAY_TRANSACTION_STATE_PROCESSED	 = 'processed';
	const QUICKPAY_TRANSACTION_STATE_CANCEL	 = 'cancel';
	
	//inherited properties
	
	protected $_location	= __FILE__;
	private   $_supportedLanguages   = array('da','de','en','fo','fr','kl','it','no','nl','pl','ru','sv');
	
	private $txnStates = array('QUICKPAY_STATUS_20000'=>'20000', 
						      'QUICKPAY_STATUS_40000'=>'40000',
							  'QUICKPAY_STATUS_40001'=>'40001',
							  'QUICKPAY_STATUS_50000'=>'50000',
							  'QUICKPAY_STATUS_50300'=>'50300'
							  );

	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	 {
		if($view == 'payment' && $task == 'notify'){
			$notify_response = file_get_contents("php://input");
			$notify_response=json_decode($notify_response,true);
			$paymentKey = $notify_response['order_id'];
			if($paymentKey){
				$prefix = JString::substr($paymentKey, 0,3);
				if($prefix !== 'PK_'){
					//get payment key from order key in payplans 1.4
					$orderId 	= XiFactory::getEncryptor()->decrypt($paymentKey);
					$order  	= PayplansOrder::getInstance($orderId);
					$paymentId 	= $order->getParam('payment_id');
					$paymentKey = XiFactory::getEncryptor()->encrypt($paymentId);
				}
				
				else {
					$paymentKey = JString::substr($paymentKey, 3);
				}
				
				JRequest::setVar('payment_key', $paymentKey, 'POST');
				return true;
			}
		}
		
		return true;
	}
	
	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		if(is_object($data)){
			$data = (array)$data;
		}
		
		$lang = XiHelperJoomla::getLanguageCode();
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  = number_format($invoice->getTotal(),2,'.','')*100;
		
		
		if(in_array($lang['language'], $this->_supportedLanguages)){
			$language= $lang['language'];
		} else {
			$language= 'en';
		}
		
		$root = JURI::root();
    	if(XiFactory::getConfig()->https == true){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}
		
		$continueurl=$root.'index.php?option=com_payplans&gateway=quickpay&view=payment&task=complete&action=success&payment_key='.$payment->getKey();
		$cancelurl=$root.'index.php?option=com_payplans&gateway=quickpay&view=payment&task=complete&action=cancel&payment_key='.$payment->getKey();
		$callbackurl=$root.'index.php?option=com_payplans&gateway=quickpay&view=payment&task=notify';
		
		 $form_params = array(
		   "version"      => "v10",
		   "merchant_id"  => $this->getAppParam('merchantid', ''),
		   "agreement_id" => $this->getAppParam('agreementid', ''),
		   "order_id"     => 'PK_'.$payment->getKey(),
		   "amount"       => $amount,
		   "currency"     => $invoice->getCurrency('isocode'),
		   "autocapture" => 1,
		   "language" => $language,
		   "continueurl" => $continueurl,
		   "cancelurl"   => $cancelurl,
		   "callbackurl" =>$callbackurl,
		 );
		
		
		$this->assign('post_url', "https://payment.quickpay.net");
		$this->assign('version', $form_params["version"]);
		$this->assign('testmode', $this->getAppParam('sandbox', false) ? '1' : '0');
		$this->assign('merchant_id', $form_params["merchant_id"]);
		$this->assign('agreement_id', $form_params["agreement_id"]);
		$this->assign('order_id', $form_params["order_id"]);
		$this->assign('amount', $form_params["amount"]);
		$this->assign('currency', $form_params["currency"]);
		$this->assign('autocapture', $form_params["autocapture"]);
		$this->assign('language', $form_params["language"]);
		$this->assign('continueurl',$form_params["continueurl"]);
		$this->assign('cancelurl', 	$form_params["cancelurl"]);
		$this->assign('callbackurl',$form_params["callbackurl"]);
		
		$this->assign('checksum', $this->sign($form_params, $this->getAppParam('payment_window_api_key', '')));
		
		

		if($invoice->isRecurring()) {
			
			$this->assign('amount',      $invoice->getTotal());
			$this->assign('invoice',     $invoice);
			$this->assign('post_url',    $root.'index.php?option=com_payplans&view=payment&task=complete&action=success&payment_key='.$payment->getKey());
			$this->assign('cancel_url',  $root.'index.php?option=com_payplans&gateway=paypalpro&view=payment&task=complete&action=cancel&payment_key='.$payment->getKey());
			$this->assign('payment_key', $payment->getKey());
			$this->assign('description', $payment->getKey());	
			return $this->_render('form_subscription');	
		}
		
        return $this->_render('form_standard');
	}
	
	
	function sign($params, $api_key) {
   		 ksort($params);
    	 $base = implode(" ", $params);
  		 return hash_hmac("sha256", $base, $api_key);
 	}

	
	
	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		$db=JFactory::getDbo();
		$logs	= PayplansHelperLogger::getLog($payment, XiLogger::LEVEL_ERROR);
		$record = array_pop($logs);			
		if($record && !empty($record)){
			$action = 'error';
		}
		
		if($action == 'cancel'){
			return true;
		}
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		if($invoice->isRecurring()){
			
			$cardnumber=str_replace(" ","",$data['cardnumber']);
			$cvd=trim($data['cvd']);
			
			$data['expirationdate'] = $data['exp_year'].$data['exp_month'];
			
			$request_subs	  = array('order_id'=> $data['order_id'], 'currency'=> $invoice->getCurrency('isocode'),'description'=> $data['description']);
			
			$request_subs_auth	  = array('amount'=>$data['amount'],'card[number]'=> $cardnumber,'card[expiration]'=> $data['expirationdate'],
								  'card[cvd]'=> $cvd);
			
			$apiKey=$this->getAppParam('api_user_api_key', '');		
			
			$sql="select * from #__payplans_transaction where invoice_id=".$invoice->getId()." and gateway_txn_id=0 and gateway_subscr_id != '' and gateway_parent_txn=0";
			$db->setQuery($sql);
			$result=$db->loadAssoc();
			//print_r($result);
			if(count($result) > 0){
				$txn_id         = 0;
				$subscr_id 		= $result['gateway_subscr_id'];
				$parent_id 		= 0;
				$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_id);
				
			} else {
				$transactions='';
			}
			
			if (!empty($transactions)) {
				foreach ($transactions as $transaction) {
					$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
					$response_subs=$transaction->getParams();
				}
				
			} else {
				$response_subs   = $this->curlRequest('/subscriptions',$request_subs, $apiKey);
			}
			
			$response_subs=json_decode($response_subs,true);
			if(isset($response_subs['id']) and $response_subs['id'] != ''){
					$txn_id         = 0;
					$subscr_id 		= $response_subs['id'];
					$parent_id 		= 0;
					$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_id);
					if (!empty($transactions)) {
						foreach ($transactions as $transaction) {
							$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
						}
					} else{
						$transaction = PayplansTransaction::getInstance();
					}
					$transaction->set('user_id', $invoice->getBuyer())
								->set('invoice_id', $invoice->getId())
								->set('payment_id', $payment->getId())
								->set('gateway_subscr_id', $subscr_id)
								->set('message', 'COM_PAYPLANS_PAYMENT_APP_QUICKPAY_SUBSCRIPTION_CREATED')
								->set('params', PayplansHelperParam::arrayToIni($response_subs))
								->save();
	
					$params = new XiParameter();
					$params->set('transaction_id', $subscr_id);
					$params->set('pending_recur_count', $this->_getRecurrenceCount($invoice));
	
					$payment->set('gateway_params', $params)
							->save();
							
				$response_subs_auth      = $this->curlRequest("/subscriptions/".$response_subs['id']."/authorize?synchronized",$request_subs_auth, $apiKey);
				$response_subs_auth=json_decode($response_subs_auth,true);
				
				if($response_subs_auth['operations'][0]['qp_status_code'] == self::QUICKPAY_STATUS_20000){
					
					$txn_id         = 0;
					$subscr_id 		= $response_subs_auth['id'];
					$parent_id 		= 0;
					$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_id);
					if (!empty($transactions)) {
						foreach ($transactions as $transaction) {
							$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
						}
					} else{
						$transaction = PayplansTransaction::getInstance();
					}
					$transaction->set('user_id', $invoice->getBuyer())
								->set('invoice_id', $invoice->getId())
								->set('payment_id', $payment->getId())
								->set('gateway_subscr_id', $subscr_id)
								->set('message', 'COM_PAYPLANS_PAYMENT_APP_QUICKPAY_SUBSCRIPTION_CREATED_AUTHORIZED')
								->set('params', PayplansHelperParam::arrayToIni($response_subs_auth))
								->save();
	
					$params = new XiParameter();
					$params->set('transaction_id', $subscr_id);
					$params->set('pending_recur_count', $this->_getRecurrenceCount($invoice));
	
					$payment->set('gateway_params', $params)
							->save();
							
					$this->_processRecurringPayment($payment, $invoice, number_format($invoice->getTotal(),2,'.','') * 100);
					
				} else {
				
					$message 	= XiText::_('COM_PAYPLANS_LOGGER_QUICKPAY_ERROR_OCCURED_IN_SUBSCRIPTION_CREATION_AUTHORIZED');
					$error 		= (string) $response_subs_auth['message'];
					//PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($error), 'PayplansPaymentFormatter', '', true);
					$getApp 	= XiFactory::getApplication();
					$getApp->enqueueMessage(XiText::_($error),'error');
					$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
				}
			} else {
					$message 	= XiText::_('COM_PAYPLANS_LOGGER_QUICKPAY_ERROR_OCCURED_IN_SUBSCRIPTION_CREATION');
					$error 		= (string) $response_subs['message'];
					PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, array($error), 'PayplansPaymentFormatter', '', true);
					$getApp 	= XiFactory::getApplication();
					$getApp->enqueueMessage(XiText::_($error),'error');
					$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
			}
		}
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	
	public function curlRequest($requestPath = null, $requestArray = null, $apiKey=null, $requestUrl = "https://api.quickpay.net/")
	{
		$ch = curl_init();
        
        $headers = array(
            'Accept-Version: v10',
            'Accept: application/json', 
        );

        if( !empty($apiKey)) 
        {
            $headers[] = 'Authorization: Basic ' . base64_encode(":{$apiKey}");
        }

        $options = array(
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
            CURLOPT_HTTPHEADER => $headers
        ); 
		
        
        curl_setopt_array( $ch, $options );
		curl_setopt( $ch, CURLOPT_URL,  $requestUrl.trim( $requestPath, '/' ));
		
		// Set the HTTP request type
 		curl_setopt( $ch, CURLOPT_CUSTOMREQUEST, 'POST' );

 		// If additional data is delivered, we will send it along with the API request
 		if( is_array( $requestArray ) && ! empty( $requestArray ) )
 		{
 			curl_setopt( $ch, CURLOPT_POSTFIELDS, http_build_query($requestArray) );
 		}
		// Store received headers in temporary memory file, remember sent headers
		$fh_header = fopen('php://temp', 'wb+');
		curl_setopt($ch, CURLOPT_WRITEHEADER, $fh_header);
		curl_setopt($ch, CURLINFO_HEADER_OUT, true);

 		// Execute the request
 		$response_data = curl_exec( $ch );
		fclose($fh_header);
		curl_close($ch);
		return $response_data;
	}
	
	
	
	function withdrawalFromSubscription($payment, $invoice, $amount)
	{ 
		$params = $payment->getGatewayParams()->toArray();
		
		$apiKey=$this->getAppParam('api_user_api_key', '');		
		
		$request_subs_rec = array('amount'=>$amount,'order_id'=> $invoice->getKey().'_'.mt_rand(0000000, 9999999),'auto_capture'=> true);	
		
		return $this->curlRequest("/subscriptions/".$params['transaction_id']."/recurring?synchronized",$request_subs_rec, $apiKey);
	}
	
	private function _processRecurringPayment(PayplansPayment $payment, $invoice, $amount)
	{
		$deductCount	= 1 ;
		$gatewayParams	= $payment->getGatewayParams();
		
		$txn_id         = 0;
		$subscr_id 		= $gatewayParams->get('transaction_id', 0);
		$parent_id 		= 0;
		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_id);
		if (!empty($transactions)) {
			foreach ($transactions as $transaction) {
				$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
				
			}
		} else{
			$transaction = PayplansTransaction::getInstance();
		}
		
		
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_subscr_id', $gatewayParams->get('transaction_id', 0))
					->set('message', 'COM_PAYPLANS_PAYMENT_APP_QUICKPAY_PAYMENT_COMPLETED_SUCCESSFULLY');
		
		// if invoice is of 0 amount then trigger wallet update event 
		if(floatval($amount) == floatval(0)){ 
			$transaction->save();
			
			//as one payment has been made so update pending_recur_count parameter of the payment 
			$gatewayParams->set('pending_recur_count', $gatewayParams->get('pending_recur_count') - $deductCount);
			$payment->set('gateway_params', $gatewayParams)
					->save();
			
		}
		
		//send request to quickpay to make a withdrawal from subscription
		$quickpayResponse = $this->withdrawalFromSubscription($payment, $invoice, $amount);
		
		$responseData=json_decode($quickpayResponse,true);
		
		
		//IMP : convert amount in proper format while creating transaction 
		$transaction->set('amount', ($responseData['operations'][1]['amount'])/100)
					->set('gateway_txn_id', isset($responseData['id'])?$responseData['id']:0)
					->set('params', PayplansHelperParam::arrayToIni($responseData));
		
		//when state and status are not for successfull transaction then set amount to 0 
		if(self::QUICKPAY_STATUS_20000 != JString::trim(JString::strtolower($responseData['operations'][1]['qp_status_code']))){
			$deductCount = 0; 
			$transaction->set('amount', 0)
						->set('message', "COM_PAYPLANS_APP_QUICKPAY_TRANSACTION_".array_search($responseData['operations'][1]['qp_status_code'], $this->txnStates));
		}
		
		$transaction->save();
		
		//as one payment has been made so update pending_recur_count parameter of the payment 
		$gatewayParams->set('pending_recur_count', $gatewayParams->get('pending_recur_count') - $deductCount);
		$payment->set('gateway_params', $gatewayParams)
				->save();
	}
	
	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$invoice_count = $invoiceCount + 1;
		
		if($invoice->isRecurring())
		{
			$recurrence_count 	= $payment->getGatewayParam('pending_recur_count');

			if($recurrence_count > 0) {
				$amount = number_format($invoice->getTotal($invoice_count),2,'.','') * 100; // amount must be in its smallest unit
				$this->_processRecurringPayment($payment, $invoice, $amount);
			}
		}
	}
	
	
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		// if same notification came more than one time
		// Check if transaction already exists,if yes then do nothing and return
		$notify_response = file_get_contents("php://input");
		$notify_response=json_decode($notify_response,true);
	    $txn_id         = isset($notify_response['id']) ? $notify_response['id'] : 0;
	    $subscr_id 		= isset($data['subscr_id']) ? $data['subscr_id'] : 0;
	    $parent_id 		= isset($data['parent_txn_id']) ? $data['parent_txn_id'] : 0;
		$exst_transaction 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_id);
    	if (!empty($exst_transaction)) {
    		foreach ($exst_transaction as $transaction) {
    			$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
    		}
    	} else {
			$transaction = PayplansTransaction::getInstance();
		}
		
		
		// get the transaction instace of lib
		$data=array_merge($data,$notify_response);
		$transaction->set('user_id', 			$payment->getBuyer())
					->set('invoice_id', 		$invoice->getId())
					->set('payment_id', 		$payment->getId())
					->set('gateway_txn_id',		$txn_id)
					->set('gateway_subscr_id', 	$subscr_id)
					->set('gateway_parent_txn', $parent_id)
					->set('params', 			PayplansHelperParam::arrayToIni($data));
					
		if('capture' === JString::trim(JString::strtolower($notify_response['operations'][1]['type']))){
			// its a normal payment			
			 $this->_processNotification($payment, $notify_response, $transaction);
			 if($notify_response['operations'][1]['qp_status_code']){
				$transaction->set('message', "COM_PAYPLANS_APP_QUICKPAY_TRANSACTION_".array_search($notify_response['operations'][1]['qp_status_code'], $this->txnStates));
			 }else {
				$transaction->set('message', 'COM_PAYPLANS_APP_QUICKPAY_TRANSACTION_NO_STATUS');
			 }
		} else {
			//set the transaction message as per status
			if($notify_response['operations'][0]['qp_status_code']){
				$transaction->set('message', "COM_PAYPLANS_APP_QUICKPAY_TRANSACTION_QUICKPAY_AUTHORIZE_STATUS_".$notify_response['operations'][0]['qp_status_code']);
			} else {
				$transaction->set('message', 'COM_PAYPLANS_APP_QUICKPAY_TRANSACTION_NO_STATUS');
			}
		}
		
		return $transaction->save();
	}
	
	protected function _processNotification($payment, $data, $transaction)
	{
		$errors = array();
		
		$amount = ($data['operations'][1]['amount'])/100;

		$transaction->set('amount', $amount);
		
		
		//check for recurring and non-recurring
		if( (self::QUICKPAY_STATUS_20000 != $data['operations'][1]['qp_status_code'])){
			$errors['error_message'] = XiText::_('COM_PAYPLANS_APP_QUICKPAY_PAYMENT_FAIL');
		}
	
		if(!empty($errors)){
				$message 			= XiText::_('COM_PAYPLANS_APP_QUICKPAY_LOGGER_ERROR_IN_PAYMENT');
				$errors['response'] = $data;
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $errors, 'PayplansPaymentFormatter', '', true);
		}
					
		return true;
	}
	
	private function _getRecurrenceCount(PayplansInvoice $invoice)
	{
		$count = $invoice->getRecurrenceCount();
		if(intval($count) === 0){
			return 9999;
		}
		
		$recurring = $invoice->isRecurring();
		if($recurring){
			
			// Recurrence Count For Regular Recurring Plan
       		if($recurring == PAYPLANS_RECURRING){
	       		$recurrence_count = $invoice->getRecurrenceCount();
       		}
       		// Recurrence Count For Recurring + Trial 1 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_1){
				$recurrence_count = $invoice->getRecurrenceCount() + 1;
       		}
       		// Recurrence Count For Recurring + Trial 2 Plan
       		if($recurring == PAYPLANS_RECURRING_TRIAL_2){
	       		$recurrence_count = $invoice->getRecurrenceCount() + 2;
       		}
		}
		return $recurrence_count;
	}
	
	public function isSupportPaymentCancellation($invoice)
	{
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}
	
	public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
		$invoice   		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);	
		$gatewayParams  = $payment->getGatewayParams();
		$transaction_id = $gatewayParams->get('transaction_id',0);

		$cancelResponse = $this->_cancelRecurring($invoice, $transaction_id);
		
		
		$responseData=json_decode($cancelResponse,true);
		
		
		
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $transaction_id)
					->set('params', PayplansHelperParam::arrayToIni($responseData));
					
		if(($responseData['operations'][1]['qp_status_code'] == self::QUICKPAY_STATUS_20000) && ($responseData['operations'][1]['type'] == self::QUICKPAY_TRANSACTION_STATE_CANCEL)){
			$transaction->set('message', 'COM_PAYPLANS_PAYMENT_APP_QUICKPAY_CANCEL_SUCCESS')
						->save();
						
			$gatewayParams->set('pending_recur_count', 0);

			$payment->set('gateway_params', $gatewayParams)
					->save();	
			parent::onPayplansPaymentTerminate($payment, $controller);		
			return $this->_render('cancel_success');	
		}
		
		$transaction->set('message', 'COM_PAYPLANS_PAYMENT_APP_QUICKPAY_CANCEL_ERROR')
					->save();
					
		$message 	= XiText::_('COM_PAYPLANS_LOGGER_QUICKPAY_ERROR_OCCURED_IN_CANCEL');
		$error 		= $responseData['operations'][1]['qp_status_code'];
		$log_id 	= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error);			
		return $this->_render('cancel_error');
		
	}
	
	protected function _cancelRecurring($invoice, $transaction_id)
	{
		
		$apiKey=$this->getAppParam('api_user_api_key', '');	
			
		return $this->curlRequest("/subscriptions/".$transaction_id."/cancel?synchronized",'', $apiKey);
		
	}
	
	public function getRecurrenceTime($expTime)
	{
		$expTime['year'] = isset($expTime['year']) ? intval($expTime['year']) : 0;
		$expTime['month'] = isset($expTime['month']) ? intval($expTime['month']) : 0;
		$expTime['day'] = isset($expTime['day']) ? intval($expTime['day']) : 0;
		$expTime['hour'] = isset($expTime['hour']) ? intval($expTime['hour']) : 0;
		
		// if only days are set then return days as it is
		if(!empty($expTime['hour'])){
			$hours = $expTime['hour'];
			
			if(!empty($expTime['day'])){
				$hours += $expTime['day'] * 24;
				
				if(!empty($expTime['month'])){
					$hours += $expTime['month'] * 30 * 24;
	
					if(!empty($expTime['year'])){
						$hours += $expTime['year'] * 365 * 24;
					}
				}
			}
			return array('period' => $hours, 'unit' => 'Hour', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
						 'message' => XiText::_('COM_PAYPLANS_APP_QUICKPAY_RECURRING_MESSAGE'));
		}
		
		// if only days are set then return days as it is
		if(!empty($expTime['day'])){
			$days = $expTime['day'];
			
			if(!empty($expTime['month'])){
				$days += $expTime['month'] * 30;

				if(!empty($expTime['year'])){
					$days += $expTime['year'] * 365;
				}
			}
			return array('period' => $days, 'unit' => 'Day', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
						 'message' => XiText::_('COM_PAYPLANS_APP_QUICKPAY_RECURRING_MESSAGE'));
		}
		
		// if months are set
		if(!empty($expTime['month'])){
			$month = $expTime['month'];
			
			if(!empty($expTime['year'])){
				$month += $expTime['year'] * 12 ;
			}
			
			return array('period' => $month, 'unit' => 'Month', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
						 'message' => XiText::_('COM_PAYPLANS_APP_QUICKPAY_RECURRING_MESSAGE'));
		}
		
		// years
		if(!empty($expTime['year'])){		
			return array('period' => $expTime['year'], 'unit' => 'Year', 'frequency' => XiText::_('COM_PAYPLANS_RECURRENCE_FREQUENCY_GREATER_THAN_ONE'),
						 'message' => XiText::_('COM_PAYPLANS_APP_QUICKPAY_RECURRING_MESSAGE'));
		}
		
		return false;
	}
}
